import 'dart:async';
import 'dart:typed_data';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'dart:isolate';
import 'dart:ui';

import 'package:saw/system_alert_window.dart';



class BackendWindow {
  factory BackendWindow() => _instance;
  static final BackendWindow _instance = BackendWindow._singleton();
  static SystemWindowPrefMode prefMode = SystemWindowPrefMode.OVERLAY;

  BackendWindow._singleton() {
    SystemAlertWindow.registerOnClickListener(callBack);
  }

  bool isShow = false;

  Future<bool?> show(
      {String title = '',
      String duration = '',
      List<WindowBtnParam>? btns,
      List<WindowTextParam>? texts}) async {
    bool? permissions =
        await SystemAlertWindow.requestPermissions(prefMode: prefMode);
    if (!permissions) {
      return permissions;
    }
    registerPort();

    await Future.forEach(btns!, (dynamic element) async {
      element.imgData =
          (await rootBundle.load(element.imgpath)).buffer.asUint8List();
    });

    SystemWindowBody body = SystemWindowBody(
      decoration: SystemWindowDecoration(
          borderRadius: 60,
          startColor: Colors.black.withOpacity(0.8),
          endColor: Colors.black.withOpacity(0.8)),
      rows: [
        EachRow(columns: [
          EachColumn(
              text: SystemWindowText(
                  text: title,
                  fontSize: 12,
                  textColor: Colors.white,
                  fontWeight: FontWeight.BOLD),
              padding: SystemWindowPadding.setSymmetricPadding(6, 8),
              decoration: SystemWindowDecoration(
                  startColor: Colors.black12, borderRadius: 25.0),
              margin: SystemWindowMargin(top: 4)),
        ], gravity: ContentGravity.CENTER),
        EachRow(
          margin: SystemWindowMargin.setSymmetricMargin(15, 0),
          columns: btns.map((e) {
            return EachColumn(
                widgetStyle: 1,
                margin: SystemWindowMargin.setSymmetricMargin(0, 10),
                imgBtn: SystemWindowImageButton(
                    width: e.size,
                    height: e.size,
                    image: e.imgData,
                    size: e.size.toDouble(),
                    tag: e.tag));
          }).toList(),
          gravity: ContentGravity.CENTER,
        ),
        EachRow(
          columns: [
            EachColumn(
              margin: SystemWindowMargin.setSymmetricMargin(0, 10),
              text: SystemWindowText(
                  text: "运动时长：$duration",
                  fontSize: 12,
                  fontWeight: FontWeight.NORMAL,
                  textColor: Colors.white),
            ),
          ],
          gravity: ContentGravity.CENTER,
        ),
        EachRow(
          margin: SystemWindowMargin.setSymmetricMargin(5, 0),
          columns: texts!
              .map((e) => EachColumn(
                    margin: SystemWindowMargin.setSymmetricMargin(0, 10),
                    text: SystemWindowText(
                        text: e.text!,
                        fontWeight: FontWeight.NORMAL,
                        fontSize: e.fontSize,
                        textColor: e.textColor),
                  ))
              .toList(),
          gravity: ContentGravity.CENTER,
        )
      ],
      padding: SystemWindowPadding(left: 16, right: 16, bottom: 10, top: 10),
    );
    if (!isShow) {
      isShow = true;
      return SystemAlertWindow.showSystemWindow(
          height: 300,
          width: 200,
          body: body,
          gravity: SystemWindowGravity.CENTER,
          prefMode: prefMode);
    }
    return SystemAlertWindow.updateSystemWindow(
        height: 300,
        width: 200,
        body: body,
        gravity: SystemWindowGravity.CENTER,
        prefMode: prefMode);
  }

  void registerPort() {
    var receivePort = ReceivePort();
    IsolateManager.registerPortWithName(receivePort.sendPort);
    receivePort.listen((message) {
      print("message====="+message);
      switch (message) {
        case 'plus':
          // GlobalVM().globalEB.fire(WindowEvent(tag: message));
          break;
        case 'subtract':
          // GlobalVM().globalEB.fire(WindowEvent(tag: message));
          break;
        case 'play':
          // GlobalVM().globalEB.fire(WindowEvent(tag: message));
          break;
        case 'next':
          // GlobalVM().globalEB.fire(WindowEvent(tag: message));
          break;
        case 'back':
          close();
          // GlobalVM().globalEB.fire(WindowEvent(tag: message));
          break;
        default:
      }
    });
  }

  Future<bool?> close() async {
    print("close------");
    // MethodChannel channel = MethodChannel("in.jvapps.system_alert_window");
    // channel.invokeMethod('backToApp');
    isShow = false;
    IsolateManager.removePortNameMapping(IsolateManager.FOREGROUND_PORT_NAME);
    return SystemAlertWindow.closeSystemWindow(prefMode: prefMode);
  }

  static void callBack(String tag) async {
    SendPort post = IsolateManager.lookupPortByName()!;
    post.send(tag);
  }
}

class WindowBtnParam {
  String? imgpath;
  int size;
  String? tag;
  late Uint8List imgData;
  WindowBtnParam({this.imgpath, this.size = 34, this.tag});
}

class WindowTextParam {
  String? text;
  double fontSize;
  Color textColor;
  WindowTextParam(
      {this.text, this.fontSize = 12, this.textColor = Colors.white});
}

class IsolateManager {
  static const FOREGROUND_PORT_NAME = "foreground_port";

  static SendPort? lookupPortByName() {
    return IsolateNameServer.lookupPortByName(FOREGROUND_PORT_NAME);
  }

  static bool registerPortWithName(SendPort port) {
    removePortNameMapping(FOREGROUND_PORT_NAME);
    return IsolateNameServer.registerPortWithName(port, FOREGROUND_PORT_NAME);
  }

  static bool removePortNameMapping(String name) {
    return IsolateNameServer.removePortNameMapping(name);
  }
}
